/*
 * include/asm-i386/i387.h
 *
 * Copyright (c) 1999 Doug Ledford <dledford@redhat.com>
 *
 * Made from various code bits pulled from other files
 * in order to put things together in a way that made
 * sense.
 *
 * FX/FPU support:
 * Copyright (c) 1999 Ingo Molnar <mingo@redhat.com>,
 *                   Gabriel Paubert <paubert@iram.es>
 */
#include <linux/config.h>

#ifndef __ASM_I386_I387_H
#define __ASM_I386_I387_H

extern int i387_hard_to_user ( struct user_i387_struct * user,
	union i387_hard_union * hard);
extern int i387_user_to_hard ( union i387_hard_union * hard,
	struct user_i387_struct * user);

/*
 * Fill out the reserved bits, treat it as an fsave struct since the
 * union makes this work for both fsave and fxsave structs.
 */
#ifdef CONFIG_X86_FX

#define i387_save_hard(x) \
do { \
	if (boot_cpu_data.x86_capability & X86_FEATURE_FXSR) { \
		__asm__ __volatile__("fxsave %0" \
				     : "=m" ((x).hard.fxsave.fxcwd)); \
	} else { \
		__asm__ __volatile__("fnsave %0; fwait;" \
				     : "=m" ((x).hard.fsave.cwd)); \
	} \
} while(0)

#define i387_restore_hard(x) \
do { \
	if (boot_cpu_data.x86_capability & X86_FEATURE_FXSR) { \
                __asm__ __volatile__("fxrstor %0" \
				     : \
				     : "m" ((x).hard.fxsave.fxcwd)); \
	} else { \
		__asm__ __volatile__("frstor %0" \
				     : \
				     :"m" ((x).hard.fsave.cwd)); \
	} \
} while(0)

#define i387_set_cwd(x,v) \
do { \
	if (boot_cpu_data.x86_capability & X86_FEATURE_FXSR) { \
		(x).fxsave.fxcwd = (short)(v); \
	} else { \
		(x).fsave.cwd = ((long)(v) | 0xffff0000); \
	} \
} while(0)

#define i387_get_swd(x,v) \
do { \
	if (boot_cpu_data.x86_capability & X86_FEATURE_FXSR) { \
		(v) = (unsigned long) (x).fxsave.fxswd; \
	} else { \
		(v) = (unsigned long) (x).fsave.swd; \
	} \
} while(0)

#define i387_set_swd(x,v) \
do { \
	if (boot_cpu_data.x86_capability & X86_FEATURE_FXSR) { \
		(x).fxsave.fxswd = (short)(v); \
	} else { \
		(x).fsave.swd = ((long)(v) | 0xffff0000); \
	} \
} while(0)

#define i387_set_twd(x,v) \
do { \
	if (boot_cpu_data.x86_capability & X86_FEATURE_FXSR) { \
		(x).fxsave.fxtwd = (short)(v); \
	} else { \
		(x).fsave.twd = ((long)(v) | 0xffff0000); \
	} \
} while(0)

static inline unsigned short fputag_KNI_to_387(unsigned char tb) {
	unsigned short tw = tb;
	tw = (tw | (tw << 4)) & 0x0f0f; /* zzzz7654zzzz3210 */
	tw = (tw | (tw << 2)) & 0x3333; /* zz76zz54zz32zz10 */
	tw = (tw | (tw << 1)) & 0x5555; /* z7z6z5z4z3z2z1z0 */
	tw = ~(tw * 3);
	return tw;
}

static inline unsigned char fputag_387_to_KNI(unsigned short tw) {
	tw = ~tw & 0x5555;		/* z7z6z5z4z3z2z1z0 */
	tw = (tw | (tw >> 1)) & 0x3333;	/* zz76zz54zz32zz10 */
	tw = (tw | (tw >> 2)) & 0x0f0f;	/* zzzz7654zzzz3210 */
	tw = (tw | (tw >> 4)) & 0x00ff; /* zzzzzzzz76543210 */
	return tw;
}

#else /* CONFIG_X86_FX */

#define i387_save_hard(x) \
do { \
	__asm__ __volatile__("fnsave %0; fwait;" \
			     : "=m" ((x).hard.fsave.cwd)); \
} while(0)

#define i387_restore_hard(x) \
do { \
	__asm__ __volatile__("frstor %0" \
			     : \
			     :"m" ((x).hard.fsave.cwd)); \
} while(0)

#define i387_set_cwd(x,v) \
do { (x).fsave.cwd = ((long)(v) | 0xffff0000); } while(0)

#define i387_get_swd(x,v) \
do { (v) = (unsigned long) (x).fsave.swd; } while(0)

#define i387_set_swd(x,v) \
do { (x).fsave.swd = ((long)(v) | 0xffff0000); } while(0)

#define i387_set_twd(x,v) \
do { (x).fsave.twd = ((long)(v) | 0xffff0000); } while(0)

#endif /* CONFIG_X86_FX */

/*
 * FPU lazy state save handling..
 */
#define save_fpu(tsk) do { \
	i387_save_hard(tsk->tss.i387); \
	tsk->flags &= ~PF_USEDFPU; \
	stts(); \
} while (0)

#define unlazy_fpu(tsk) do { \
	if (tsk->flags & PF_USEDFPU) \
		save_fpu(tsk); \
} while (0)

#define clear_fpu(tsk) do { \
	if (tsk->flags & PF_USEDFPU) { \
		tsk->flags &= ~PF_USEDFPU; \
		stts(); \
	} \
} while (0)

#endif /* __ASM_I386_I387_H */
